@@ -5,8 +5,32 @@ import pickle
 import gc
 
 from graph_kmer_index import kmer_hash_to_sequence, sequence_to_kmer_hash
+from npstructures import Counter, HashTable
 from Bio.Seq import Seq
 
+from .multi_value_hashtable import MultiValueHashTable
+
+class CounterKmerIndex:
+    def __init__(self, kmers, nodes, counter):
+        self.kmers = kmers
+        self.nodes = nodes
+        self.counter = counter
+
+    @classmethod
+    def from_kmer_index(cls, kmer_index, modulo):
+        kmers = kmer_index._kmers.astype(np.int64)
+        nodes = kmer_index._nodes
+        unique_kmers = np.unique(kmers)
+        counter = Counter(unique_kmers, np.zeros_like(unique_kmers), mod=modulo)
+        return cls(kmers, nodes, counter)
+
+    def count_kmers(self, kmers):
+        self.counter.count(kmers.astype(np.int64))
+
+    def get_node_counts(self):
+        return np.bincount(self.nodes, self.counter[self.kmers])
+
+
 
 class MinimalKmerIndex:
     def __init__(self, hashes_to_index, n_kmers, nodes, kmers, modulo):
@@ -14,7 +38,7 @@ class MinimalKmerIndex:
         self._n_kmers = n_kmers.astype(np.uint32)
         self._nodes = nodes.astype(np.uint32)
         self._kmers = kmers
-        self._modulo = modulo
+        self._modulo = np.int64(modulo)
 
     def max_node_id(self):
         return np.max(self._nodes)
@@ -73,6 +97,60 @@ class MinimalKmerIndex:
         return object
 
 
+
+class KmerIndex2:
+    # same as CollisionFreeKmerIndex, but with startnode/offset in stead of ref_offset
+    def __init__(self, data, frequencies=None):
+        self._data = data
+        self._frequencies = frequencies
+
+    def get_start_nodes(self, kmer):
+        return self._data[kmer]["start_nodes"]
+
+    def get_start_offsets(self, kmer):
+        return self._data[kmer]["start_offsets"]
+
+    def get_nodes(self, kmer):
+        return self._data[kmer]["nodes"]
+
+    def get_all_kmers(self):
+        return self._data.get_all_keys()
+
+
+    def get_kmer_frequency(self, kmer):
+        assert self._frequencies is not None, "Frequencies not set"
+        return self._frequencies[kmer]
+
+    @classmethod
+    def from_flat_kmers(cls, flat_kmers, modulo=None, skip_frequencies=False):
+        hash_table = MultiValueHashTable.from_keys_and_values(flat_kmers._hashes,
+                                         {"nodes": flat_kmers._nodes,
+                                          "start_nodes": flat_kmers._start_nodes,
+                                          "start_offsets": flat_kmers._start_offsets,
+                                          "allele_frequencies": flat_kmers._allele_frequencies},
+                                         mod=modulo)
+
+        index = cls(hash_table)
+        if not skip_frequencies:
+            index.count_unique_kmer_occurences()
+
+        return index
+
+    def count_unique_kmer_occurences(self):
+        unique_kmers = self._data.get_unique_keys()
+        counts = np.zeros_like(unique_kmers)
+
+        # count unique entries (unique start node/offset) for each kmer
+        for i, kmer in enumerate(unique_kmers):
+            n = len(set((node, offset) for node, offset in
+                        zip(self.get_start_nodes(kmer), self.get_start_offsets(kmer))))
+            counts[i] = n
+
+        self._frequencies = HashTable(unique_kmers, counts)
+
+
+
+
 class CollisionFreeKmerIndex:
     properties = {
             "_hashes_to_index",
@@ -93,7 +171,7 @@ class CollisionFreeKmerIndex:
         self._ref_offsets = _ref_offsets
         self._kmers = _kmers  # Actual numeric kmers (not hashes of numeric kmers) at each position
                              # used to filter out collisions
-        self._modulo = _modulo
+        self._modulo = int(_modulo)
         if _frequencies is None:
             self._frequencies = 0
         else:
@@ -126,6 +204,15 @@ class CollisionFreeKmerIndex:
     def max_node_id(self):
         return np.max(self._nodes)
 
+    def convert_to_int32(self):
+        self._hashes_to_index = self._hashes_to_index.astype(np.int32)
+        self._nodes = self._nodes.astype(np.int32)
+        self._n_kmers = self._n_kmers.astype(np.int32)
+        self._modulo = np.uint64(self._modulo)
+
+    def remove_ref_offsets(self):
+        self._ref_offsets = np.array([0])
+
     def set_frequencies_using_other_index(self, other, multiplier=1, min_frequency=1):
         unique = np.unique(self._kmers)
         for i, kmer in enumerate(unique):
@@ -169,10 +256,13 @@ class CollisionFreeKmerIndex:
             assert count > 0, "Count is not > 0 for kmer %d, start, end: %d,%d. Ref offsets: %s" % (kmer, start, end, self._ref_offsets[hit_positions + start])
             self._frequencies[hit_positions + start] = count
 
+    def __contains__(self, item):
+        return self.get(int(item), 100000000000)[0] is not None
+
     def get(self, kmer, max_hits=10):
         hash = kmer % self._modulo
-        position = self._hashes_to_index[hash]
-        n_hits = self._n_kmers[hash]
+        position = self._hashes_to_index[int(hash)]
+        n_hits = self._n_kmers[int(hash)]
         start = position
         end = position + n_hits
         hit_positions = np.where(self._kmers[start:end] == kmer)[0]
@@ -277,12 +367,16 @@ class CollisionFreeKmerIndex:
         except FileNotFoundError:
             data = np.load(file_name)
 
-        if "allele_frequencies" in data:
-            allele_frequencies = data["allele_frequencies"]
-        else:
-            allele_frequencies = np.zeros(len(data["ref_offsets"]))
+        try:
+            if "allele_frequencies" in data:
+                allele_frequencies = data["allele_frequencies"]
+            else:
+                allele_frequencies = np.zeros(len(data["ref_offsets"]))
 
-        return cls(data["hashes_to_index"], data["n_kmers"], data["nodes"], data["ref_offsets"], data["kmers"], data["modulo"], data["frequencies"], allele_frequencies)
+            return cls(data["hashes_to_index"], data["n_kmers"], data["nodes"], data["ref_offsets"], data["kmers"], data["modulo"], data["frequencies"], allele_frequencies)
+        except AttributeError:
+            logging.info("Keys are %s" % (data.keys()))
+            raise
 
     @classmethod
     def from_flat_kmers(cls, flat_kmers, modulo=452930477, skip_frequencies=False, skip_singletons=False):
@@ -314,7 +408,7 @@ class CollisionFreeKmerIndex:
             logging.info("Hashes: %s" % hashes)
             raise
 
-        lookup = np.zeros(modulo, dtype=np.int)
+        lookup = np.zeros(modulo, dtype=np.int32)
         lookup[unique_hashes] = unique_entry_positions
         n_entries = np.ediff1d(unique_entry_positions, to_end=len(nodes)-unique_entry_positions[-1])
         n_kmers = np.zeros(modulo, dtype=np.uint32)
