@@ -1,7 +1,10 @@
+import logging
+logging.basicConfig(level=logging.INFO, format='%(asctime)s %(levelname)s: %(message)s')
 import sys
 import argparse
-import logging
+import itertools
 #from multiprocessing import shared_memory
+import time
 from multiprocessing import Pool, Process
 import numpy as np
 from itertools import repeat
@@ -9,7 +12,6 @@ from pyfaidx import Fasta
 
 from .collision_free_kmer_index import CollisionFreeKmerIndex
 
-logging.basicConfig(level=logging.INFO, format='%(asctime)s %(levelname)s: %(message)s')
 from obgraph import Graph
 from .snp_kmer_finder import SnpKmerFinder
 import pickle
@@ -20,9 +22,20 @@ from .reference_kmer_index import ReferenceKmerIndex
 from pathos.multiprocessing import Pool
 from obgraph.variants import VcfVariants
 from .unique_variant_kmers import UniqueVariantKmersFinder
-from shared_memory_wrapper.shared_memory import to_shared_memory, from_shared_memory, remove_shared_memory_in_session
+from shared_memory_wrapper.shared_memory import to_shared_memory, from_shared_memory, remove_shared_memory_in_session, to_file, from_file, object_to_shared_memory, object_from_shared_memory
+from shared_memory_wrapper import get_shared_pool, close_shared_pool
 from obgraph.variant_to_nodes import VariantToNodes, NodeToVariants
 from obgraph.haplotype_matrix import HaplotypeMatrix
+from npstructures import HashTable, Counter
+from .npset import NpSet
+from .collision_free_kmer_index import CounterKmerIndex
+from .kmer_finder import DenseKmerFinder
+from .collision_free_kmer_index import KmerIndex2
+from .critical_graph_paths import CriticalGraphPaths
+from shared_memory_wrapper import to_file, from_file
+from obgraph.position_id import PositionId
+
+from kmer_mapper.util import log_memory_usage_now
 
 
 def main():
@@ -66,7 +79,7 @@ def create_index_single_thread(args, interval=None):
         skip_kmers_with_nodes = set(f._nodes)
 
     finder = SnpKmerFinder(graph, k=args.kmer_size, spacing=args.spacing,
-                           include_reverse_complements=args.include_reverse_complement,
+                           include_reverse_complements=False,  # always false, adds revcomp in the end if chosen
                            pruning=args.pruning,
                            max_kmers_same_position=args.max_kmers_same_position,
                            max_frequency=args.max_frequency,
@@ -81,6 +94,14 @@ def create_index_single_thread(args, interval=None):
                            reference=reference)
 
     kmers = finder.find_kmers()
+
+    if args.include_reverse_complement:
+        logging.info("Adding reverse complements")
+        log_memory_usage_now("before adding reverse complements")
+        kmers_revcomp = kmers.get_reverse_complement_flat_kmers(args.kmer_size)
+        log_memory_usage_now("after adding reverse complements")
+        kmers = FlatKmers.from_multiple_flat_kmers([kmers, kmers_revcomp])
+
     return kmers
 
 def create_index(args):
@@ -95,14 +116,15 @@ def create_index(args):
         kmers = create_index_single_thread(args)
         kmers.to_file(args.out_file_name)
     else:
-        logging.info("Making pool with %d workers" % args.threads)
+        n_jobs = args.threads * 10
+        logging.info("Making pool with %d workers" % n_jobs)
         pool = Pool(args.threads)
         genome_size = args.genome_size
         n_total_start_positions = genome_size // args.spacing
-        n_positions_each_process = n_total_start_positions // args.threads
+        n_positions_each_process = n_total_start_positions // n_jobs
         logging.info("Using genome size %d. Will process %d genome positions in each process." % (genome_size, n_positions_each_process))
         intervals = []
-        for i in range(args.threads):
+        for i in range(n_jobs):
             start_position = n_positions_each_process * i * args.spacing
             end_position = n_positions_each_process * (i+1) * args.spacing
             intervals.append((start_position, end_position))
@@ -274,59 +296,74 @@ def run_argument_parser(args):
     subparser.set_defaults(func=make_reference_kmer_index)
 
     def make_unique_variant_kmers_single_thread(variants, args):
-        r = args.shared_memory_unique_id
-        variant_to_nodes = from_shared_memory(VariantToNodes, "variant_to_nodes_shared"+r)
-        kmer_index = from_shared_memory(CollisionFreeKmerIndex, "kmer_index_shared"+r)
-        graph = from_shared_memory(Graph, "graph_shared"+r)
+        args = object_from_shared_memory(args)
+        #variant_to_nodes = from_shared_memory(VariantToNodes, "variant_to_nodes_shared"+r)
+        variant_to_nodes = args["variant_to_nodes"]
+
+        #kmer_index = from_shared_memory(CollisionFreeKmerIndex, "kmer_index_shared"+r)
+        kmer_index = args["kmer_index"]
+        if kmer_index is None:
+            kmer_index = args["kmer_counter"]
+
+        #graph = from_shared_memory(Graph, "graph_shared"+r)
+        graph = args["graph"]
+
         #graph = Graph.from_file(args.graph)
         logging.info("Reading all variants")
 
         node_to_variants = None
         haplotype_matrix = None
+        """
         if args.haplotype_matrix is not None:
             haplotype_matrix = HaplotypeMatrix.from_file(args.haplotype_matrix)
             node_to_variants = NodeToVariants.from_file(args.node_to_variants)
+        """
 
-        finder = UniqueVariantKmersFinder(graph, variant_to_nodes, variants, args.kmer_size, args.max_variant_nodes,
+        use_dense_kmer_finder = False
+        if args["use_dense_kmer_finder"]:
+            use_dense_kmer_finder = True
+            logging.info("Will use dense kmer finder")
+
+        finder = UniqueVariantKmersFinder(graph, variant_to_nodes, variants, args["kmer_size"], args["max_variant_nodes"],
                                           kmer_index_with_frequencies=kmer_index, haplotype_matrix=haplotype_matrix,
                                           node_to_variants=node_to_variants,
-                                          do_not_choose_lowest_frequency_kmers=args.do_not_choose_lowest_frequency_kmers)
+                                          do_not_choose_lowest_frequency_kmers=args["do_not_choose_lowest_frequency_kmers"],
+                                          use_dense_kmer_finder=use_dense_kmer_finder,
+                                          position_id_index=args["position_id_index"]
+                                          )
         flat_kmers = finder.find_unique_kmers()
         return flat_kmers
 
     def make_unique_variant_kmers(args):
-        args.shared_memory_unique_id = str(np.random.randint(0,10e15))
-        r = args.shared_memory_unique_id
-        logging.info("Reading kmer index")
-        kmer_index = CollisionFreeKmerIndex.from_file(args.kmer_index)
-        to_shared_memory(kmer_index, "kmer_index_shared"+r)
-        logging.info("Reading variant to nodes")
-        variant_to_nodes = VariantToNodes.from_file(args.variant_to_nodes)
-        to_shared_memory(variant_to_nodes, "variant_to_nodes_shared"+r)
-        logging.info("REading graph")
-        graph = Graph.from_file(args.graph)
-        to_shared_memory(graph, "graph_shared"+r)
+        args = vars(args)
+        args_orig = args
+        args.pop("func")  # necessary for putting args in shared memory
+
         logging.info("Reading all variants")
-        variants = VcfVariants.from_vcf(args.vcf, skip_index=True, make_generator=True)
-        variants = variants.get_chunks(chunk_size=args.chunk_size)
-        pool = Pool(args.n_threads)
+        variants = VcfVariants.from_vcf(args["vcf"], skip_index=True, make_generator=True)
+        variants = variants.get_chunks(chunk_size=args["chunk_size"])
+        pool = Pool(args["n_threads"])
 
         all_flat_kmers = []
+        args = object_to_shared_memory(args)
         for flat_kmers in pool.starmap(make_unique_variant_kmers_single_thread, zip(variants, repeat(args))):
             all_flat_kmers.append(flat_kmers)
 
         logging.info("Merge all flat kmers")
         merged_flat = FlatKmers.from_multiple_flat_kmers(all_flat_kmers)
-        merged_flat.to_file(args.out_file_name)
-        logging.info("Wrote to file %s" % args.out_file_name)
+        merged_flat.to_file(args_orig["out_file_name"])
+        logging.info("Wrote to file %s" % args_orig["out_file_name"])
 
     subparser = subparsers.add_parser("make_unique_variant_kmers", help="Make a reverse variant index lookup to unique kmers on that variant")
-    subparser.add_argument("-g", "--graph", required=True)
-    subparser.add_argument("-V", "--variant_to_nodes", required=True)
+    subparser.add_argument("-g", "--graph", required=True, type=Graph.from_file)
+    subparser.add_argument("-V", "--variant_to_nodes", required=True, type=VariantToNodes.from_file)
     subparser.add_argument("-N", "--node-to-variants", required=False)
     subparser.add_argument("-H", "--haplotype-matrix", required=False)
     subparser.add_argument("-k", "--kmer-size", required=True, type=int)
-    subparser.add_argument("-i", "--kmer-index", required=True, help="Kmer index used to check frequency of kmers in genome")
+    subparser.add_argument("-i", "--kmer-index", required=False, help="Kmer index used to check frequency of kmers in genome", type=CollisionFreeKmerIndex.from_file)
+    subparser.add_argument("-I", "--kmer-counter", required=False, help="Kmer index used to check frequency of kmers in genome", type=from_file)
+    subparser.add_argument("-p", "--position-id-index", required=False, type=from_file)
+    subparser.add_argument("-D", "--use-dense-kmer-finder", required=False, type=bool, default=False)
     subparser.add_argument("-o", "--out-file-name", required=True)
     subparser.add_argument("-v", "--vcf", required=True)
     subparser.add_argument("-t", "--n-threads", required=False, default=1, type=int)
@@ -390,6 +427,155 @@ def run_argument_parser(args):
     subparser.add_argument("-f", "--frequencies", required=True)
     subparser.set_defaults(func=set_allele_frequencies)
 
+
+    def create_counter_index(args):
+        kmer_index = CollisionFreeKmerIndex.from_file(args.kmer_index)
+        counter_index = CounterKmerIndex.from_kmer_index(kmer_index, args.modulo)
+        to_file(counter_index, args.out_file_name)
+
+
+    subparser = subparsers.add_parser("create_counter_index")
+    subparser.add_argument("-i", "--kmer-index", required=True)
+    subparser.add_argument("-o", "--out-file-name", required=True)
+    subparser.add_argument("-m", "--modulo", required=False, default=200000033, type=int)
+    subparser.set_defaults(func=create_counter_index)
+
+
+    def index_single_thread(data):
+        args, chunk = data
+        args = object_from_shared_memory(args)
+        logging.info("Processing chunk %s" % str(chunk))
+        log_memory_usage_now(str(chunk))
+
+        t = time.perf_counter()
+        kmer_finder = DenseKmerFinder(args["graph"], args["kmer_size"], critical_graph_paths=args["critical_graph_paths"],
+                                      position_id=args["position_id"],
+                                      max_variant_nodes=args["max_variant_nodes"],
+                                      only_save_one_node_per_kmer=True,
+                                      start_at_critical_path_number=chunk[0],
+                                      stop_at_critical_path_number=chunk[1],
+                                      whitelist=args["whitelist"])
+        kmer_finder.find()
+        logging.info("Took %d sec to find kmers" % (time.perf_counter()-t))
+        res = kmer_finder.get_flat_kmers(v="1")
+        if len(np.where(res._hashes == 979349781162838629)[0]) > 0:
+            logging.info("FOUND MATCH! %d/%d" % (chunk[0], chunk[1]))
+
+        return res
+
+    def index(args):
+        pool = get_shared_pool(args.n_threads)
+
+        if args.critical_graph_paths is None:
+            logging.info("Making critical graph paths since not specified")
+            args.critical_paths = CriticalGraphPaths.from_graph(graph, args.k)
+
+        args.position_id = PositionId.from_graph(args.graph) if args.position_id is not None else None
+
+        args = vars(args)
+        args.pop("func")
+        log_memory_usage_now("Before shared memory")
+        args_shared = object_to_shared_memory(args)
+        log_memory_usage_now("After shared memory")
+        critical_paths = args["critical_graph_paths"]
+
+        n_chunks = args["n_threads"]*20
+        if n_chunks >= len(critical_paths):
+            n_chunks = len(critical_paths)
+
+        assert n_chunks <= len(critical_paths), "Too many threads for data. Try with fewer"
+
+        logging.info("There are %d critical paths" % len(critical_paths))
+        logging.info("Will process these in %d chunks" % n_chunks)
+        n_paths_per_chunk = len(critical_paths) // n_chunks
+        logging.info("N paths per chunk: %d" % n_paths_per_chunk)
+
+        starting_indexes = list(range(0, len(critical_paths), n_paths_per_chunk))
+        ending_indexes = starting_indexes[1:] + [len(critical_paths)]
+        chunks = list((s, e) for s, e in zip(starting_indexes, ending_indexes))
+
+        logging.info("Will process chunks (from,to): %s" % chunks)
+
+        t = time.perf_counter()
+        flat_kmers = []
+        for i, results in enumerate(pool.imap(index_single_thread, zip(itertools.repeat(args_shared), chunks))):
+            flat_kmers.append(results)
+            logging.info("Done with chunk. Found %d kmers" % (len(results._hashes)))
+            log_memory_usage_now("Done with chunk %d" % i)
+
+        logging.info("Time spent to make indexes: %.2f" % (time.perf_counter()-t))
+        close_shared_pool()
+
+        flat_kmers = FlatKmers.from_multiple_flat_kmers(flat_kmers)
+        logging.info("N kmers in flat kmers: %d" % len(flat_kmers._hashes))
+        if args["include_reverse_complement"]:
+            logging.info("Adding reverse complements")
+            revcomp_flat_kmers = flat_kmers.get_reverse_complement_flat_kmers(args["kmer_size"])
+            flat_kmers = FlatKmers.from_multiple_flat_kmers([flat_kmers, revcomp_flat_kmers])
+            logging.info("N kmers in flat kmers after revcomp added: %d" % len(flat_kmers._hashes))
+
+        flat_kmers.to_file(args["out_file_name"])
+        logging.info("Wrote final FlatKmers to %s" % args["out_file_name"])
+
+
+    subparser = subparsers.add_parser("index")
+    subparser.add_argument("-g", "--graph", required=True, type=Graph.from_file)
+    subparser.add_argument("-c", "--critical_graph_paths", required=False, type=from_file, help="Will be created if not specified")
+    subparser.add_argument("-p", "--position_id", required=False, type=from_file, help="Will be created if not specified")
+    subparser.add_argument("-k", "--kmer-size", type=int, default=31, required=False)
+    subparser.add_argument("-o", "--out-file-name", required=True)
+    subparser.add_argument("-t", "--n-threads", required=False, type=int, default=1)
+    #subparser.add_argument("-w", "--whitelist", required=False, type=lambda e: NpSet(FlatKmers.from_file(e)._hashes), help="Only store kmers that are in the whitelist")
+    subparser.add_argument("-w", "--whitelist", required=False, type=lambda e: CollisionFreeKmerIndex.from_file(e), help="Only store kmers that are in the whitelist")
+    subparser.add_argument("-r", "--include-reverse-complement", required=False, type=bool, default=False)
+    subparser.add_argument("-O", "--only-save-one-node-per-kmer", required=False, type=bool, default=False)
+    subparser.add_argument("-v", "--max-variant-nodes", required=False, type=int, default=5, help="Max variant nodes allowed in kmer.")
+    subparser.set_defaults(func=index)
+
+
+    def find_critical_paths(args):
+        from .critical_graph_paths import CriticalGraphPaths
+        graph = Graph.from_file(args.graph)
+        critical_paths = CriticalGraphPaths.from_graph(graph, args.kmer_size)
+        critical_paths._make_index()
+        to_file(critical_paths, args.out_file_name)
+        logging.info("Wrote to file %s" % args.out_file_name)
+
+    subparser = subparsers.add_parser("find_critical_paths")
+    subparser.add_argument("-g", "--graph", required=True)
+    subparser.add_argument("-k", "--kmer-size", type=int, default=31, required=False)
+    subparser.add_argument("-o", "--out-file-name", required=True)
+    subparser.set_defaults(func=find_critical_paths)
+
+
+    def add_reverse_complements(args):
+        flat = FlatKmers.from_file(args.flat_kmers)
+        rev_comp_flat = flat.get_reverse_complement_flat_kmers(k=args.kmer_size)
+        flat = FlatKmers.from_multiple_flat_kmers([flat, rev_comp_flat])
+        flat.to_file(args.out_file_name)
+        logging.info("Saved new flat kmers with reverse complements to %s" % args.out_file_name)
+
+    subparser = subparsers.add_parser("add_reverse_complements")
+    subparser.add_argument("-f", "--flat-kmers", required=True)
+    subparser.add_argument("-o", "--out-file-name", required=True)
+    subparser.add_argument("-k", "--kmer-size", type=int, required=True)
+    subparser.set_defaults(func=add_reverse_complements)
+
+
+    def count_kmers(args):
+        from .kmer_counter import KmerCounter
+        counter = KmerCounter.from_flat_kmersv2(args.flat_kmers, args.modulo, args.subsample_ratio)
+        to_file(counter, args.out_file_name)
+        logging.info("Wrote counter to %s" % args.out_file_name)
+
+    subparser = subparsers.add_parser("count_kmers")
+    subparser.add_argument("-f", "--flat-kmers", required=True, type=FlatKmers.from_file)
+    subparser.add_argument("-o", "--out-file-name", required=True)
+    subparser.add_argument("-m", "--modulo", required=False, type=int, default=200000033)
+    subparser.add_argument("-s", "--subsample-ratio", required=False, type=int, default=1, help="1 to keep every kmer, 2 for every other etc")
+    subparser.set_defaults(func=count_kmers)
+
+
     if len(args) == 0:
         parser.print_help()
         sys.exit(1)
