@@ -4,6 +4,25 @@ from collections import defaultdict
 from .kmer_hashing import kmer_hashes_to_reverse_complement_hash
 
 
+class FlatKmers2:
+    def __init__(self, hashes, start_nodes, start_offsets, nodes, allele_frequencies):
+        assert len(hashes) == len(nodes)
+        assert len(start_nodes) == len(nodes)
+        assert len(start_offsets) == len(start_nodes)
+
+        self._hashes = hashes
+        self._nodes = nodes
+        self._start_nodes = start_nodes
+        self._start_offsets = start_offsets
+
+        if allele_frequencies is None:
+            logging.info("Allele frequencies not provided. Setting all to 1.0")
+            self._allele_frequencies = np.zeros(len(self._hashes), dtype=np.single) + 1.0
+        else:
+            self._allele_frequencies = allele_frequencies
+
+
+
 class FlatKmers:
     def __init__(self, hashes, nodes, ref_offsets=None, allele_frequencies=None):
         assert len(hashes) == len(nodes)
@@ -57,7 +76,7 @@ class FlatKmers:
             ref_offsets = np.array(ref_offsets, np.uint64)
 
         logging.info("Done making flat kmers")
-        return FlatKmers(np.array(hashes, dtype=np.uint64), np.array(nodes, np.uint32), ref_offsets, np.array(allele_frequencies, dtype=np.uint16))
+        return FlatKmers(np.array(hashes, dtype=np.uint64), np.array(nodes, np.uint32), ref_offsets, np.array(allele_frequencies, dtype=np.single))
 
     def sum_of_kmer_frequencies(self, kmer_index_with_frequencies):
         return sum([0] + [max(1, kmer_index_with_frequencies.get_frequency(int(kmer))) for kmer in self._hashes])
