@@ -7,6 +7,7 @@ import numpy as np
 from .flat_kmers import FlatKmers, letter_sequence_to_numeric
 from Bio.Seq import Seq
 from collections import defaultdict
+from kmer_mapper.util import log_memory_usage_now
 
 
 def kmer_hash_to_sequence(hash, k):
@@ -229,7 +230,7 @@ class SnpKmerFinder:
         self._last_ref_pos_added = self._current_ref_offset
 
         self._kmers_found += 1
-        if len(self.kmers_found) < 200:
+        if len(self.kmers_found) < 500:
             # Only add to this when there is little data, only used for testing and debugging
             #loggign.info("------Added kmer with nodes %s" % nodes)
             self.kmers_found.append((kmer, nodes, self._current_ref_offset, hash))
@@ -326,17 +327,19 @@ class SnpKmerFinder:
     def find_kmers_from_linear_ref_position(self, pos):
         self._find_all_variant_kmers_from_position(pos)
 
-    def get_flat_kmers(self):
+    def get_flat_kmers(self, v=None):
         return FlatKmers(np.array(self._hashes, dtype=np.uint64), np.array(self._nodes, np.uint32), np.array(self._ref_offsets, np.uint64), np.array(self._allele_frequencies, np.single))
 
     def find_kmers_on_linear_reference(self):
         #assert self.spacing == 1, "Finding kmers on linear reference is only possible when spacing is 1"
         #loggign.info("Fetching reference sequence between %d and %d" % (self._start_position, self._end_position+self.k))
         reference_sequence = str(self.reference[self._start_position:self._end_position+self.k])
+        log_memory_usage_now("afte ref sequence to string")
         assert len(reference_sequence) > 0
         #loggign.info("Fetching kmers")
         from .read_kmers import ReadKmers
         kmers = ReadKmers.get_kmers_from_read_dynamic(reference_sequence, np.power(4, np.arange(0, self.k)))
+        log_memory_usage_now("got kmers from read")
         kmers = kmers[::self.spacing]
         #loggign.info("Done fetching kmers")
 
